import java.math.BigDecimal;
import java.math.MathContext;

public class CartPoint
{
  final BigDecimal x,y,z;

  public CartPoint(BigDecimal x, BigDecimal y, BigDecimal z)
  {
    this.x = x;
    this.y = y;
    this.z = z;
    
  }

  public CartPoint(String s)
  {
    String split[] = s.split(",");
    x = new BigDecimal(split[0].trim());
    y = new BigDecimal(split[1].trim());
    z = new BigDecimal(split[2].trim());
  }

  public CartPoint(String x, String y, String z)
  {
    this(new BigDecimal(x), new BigDecimal(y), new BigDecimal(z));
  }

  public BigDecimal getDist(CartPoint b)
  {

    BigDecimal dx = b.x.subtract(x);
    BigDecimal dy = b.y.subtract(y);
    BigDecimal dz = b.z.subtract(z);

    BigDecimal s = BigDecimal.ZERO;
    s = s.add( dx.pow(2));
    s = s.add( dy.pow(2));
    s = s.add( dz.pow(2));

    return s.sqrt(new MathContext(200));
  }

  public RadPoint getRad()
  {
    double theta = Math.atan2(y.doubleValue(), x.doubleValue());
    double phi = Math.atan2(Math.sqrt(x.pow(2).doubleValue() + y.pow(2).doubleValue()),z.doubleValue());

    // Note: discards radius and puts back on unit sphere
    return new RadPoint(theta, phi);
  }

  // Get direction from this point towards b
  public CartPoint getDirection(CartPoint b)
  {
    return new CartPoint(b.x.subtract(x), b.y.subtract(y), b.z.subtract(z));
  }

  public CartPoint reverse()
  {
    return new CartPoint(x.negate(), y.negate(), z.negate());
  }
  public CartPoint multi(double m)
  {
    BigDecimal mu = BigDecimal.valueOf(m);
    return new CartPoint(x.multiply(mu), y.multiply(mu), z.multiply(mu));
  }

  public CartPoint add(CartPoint o)
  {
    return new CartPoint(x.add(o.x), y.add(o.y), z.add(o.z));
  }
  public String toString()
  {
    return String.format("C{%f %f %f}",x,y,z);
  }



}
